# -*- coding:utf-8 -*-
import sys
from functools import wraps

# from .exceptions import NotSupprotedYetException
# If used , it will cause circulation import
"""
File "/home/hzcortex/FBRank/FBRank/parse/League.py", line 13, in <module>
    from FBRank.utils.exceptions import IllegalArgumentException, NotSupprotedYetException
  File "/home/hzcortex/FBRank/FBRank/utils/exceptions.py", line 2, in <module>
    from .utils import github_url, connect_url
  File "/home/hzcortex/FBRank/FBRank/utils/utils.py", line 5, in <module>
    from .exceptions import NotSupprotedYetException
ImportError: cannot import name 'NotSupprotedYetException'

"""

PY2 = sys.version[0] == '2'


# decorator for check-name
# For Club Check
def check_before(attr='name'):
    from .exceptions import NotSupprotedYetException
    def wrapped_func(func):
        @wraps(func)
        def wrapped(self, *args, **kwargs):
            # print(func.__class__.__name__)
            if (getattr(self, attr).lower() not in club_transformat.keys()):
                raise NotSupprotedYetException
            return func(self, *args, **kwargs)

        return wrapped

    return wrapped_func


# relationship

EPL_League_transformat = {
    "利物浦":"Liverpool",
    "曼城":"Man City",
    "热刺":"Spurs",
    "阿森纳":"Arsenal",
    "曼联":"Man Utd",
    "谢菲联":"Sheffield",
    "切尔西":"Chelsea",
    "沃特福德":"Watford",
    "狼队":"Wolves",
    "西汉姆":"West Ham",
    "伯恩茅斯":"AFC Bournemouth",
    "埃弗顿":"Everton",
    "莱斯特":"Leicester",
    "水晶宫":"Crystal Palace",
    "伯恩利":"Burnley",
    "纽卡斯尔":"Newcastle",
    "布莱顿":"Brighton",
    "卡迪夫城":"Cardiff",
    "南安普顿":"Southampton",
    "富勒姆":"Fulham",
    "哈德斯菲尔德":"Huddersfield",
    "卢顿":"Luton Town",
    "巴恩斯利":"Barnsley",
    "桑德兰":"Sunderland",
    "朴茨茅斯":"Portsmouth",
    "查尔顿":"Charlton Athletic",
    "唐卡斯特":"Doncaster Rovers",
    "彼得堡联":"Peterborough United",
    "福利特":"Fleetwood Town",
    "布莱克浦":"Blackpool",
    "考文垂":"Coventry City",
    "伯顿":"Burton Albion",
    "威科姆":"Wycombe Wanderers",
    "普利茅斯":"Southend United",
    "斯肯索普":"Plymouth Argyle",
    "南安联":"Accrington Stanley",
    "阿克宁顿":"Scunthorpe United",
    "吉灵汉姆":"Gillingham",
    "沃尔索尔":"Walsall",
    "牛津联":"Shrewsbury Town",
    "什鲁斯":"Oxford United",
    "罗奇代尔":"Bristol Rovers",
    "布流浪":"Rochdale",
    "布拉德福":"Bradford City",
    "温布尔登":"AFC Wimbledon"
}

league_transformat = {
    ('英超', 'EPL','Premier League', 'Premiere League', '英国', '英格兰'): 'EPL',
    ('德甲', 'Bundesliga', 'Bunde', '德国'): 'Bunde',
    ('西甲', 'Liga BBva', '西班牙'): 'La Liga',
    ('意甲', 'Serie A', '意大利'): 'Serie A'
}

league_configure = {
    'EPL': {
        'rank_url': 'http://soccer.hupu.com/table/England.html',
        'news_url': 'http://sports.sina.com.cn/g/premierleague/'
    },
    'Bunde': {
        'rank_url': 'http://soccer.hupu.com/table/Germany.html',
        'news_url': 'http://sports.sina.com.cn/g/bundesliga/'
    },
    'La Liga': {
        'rank_url': 'http://soccer.hupu.com/table/Spain.html',
        'news_url': 'http://sports.sina.com.cn/g/laliga/'
    },
    'Serie A': {
        'rank_url': 'http://soccer.hupu.com/table/Italy.html',
        'news_url': 'http://sports.sina.com.cn/g/seriea/'
    }
}

club_transformat = {
    ('manchester united', '曼联', '曼彻斯特联'): 'manutd'
}

club_url = {

}

football_url = {

}

connect_url = 'iamwanghz@gmail.com'
github_url = 'https://github.com/FBRank'

league_news_pattern = r'<h2><a class="f074" href="(.*?)".*?>(.*?)</a>'



# -*- coding:utf-8 -*-
"""
parse web content about league,now it includes:premier league,Liga BBVA
"""

import re
from collections import defaultdict

import requests
from bs4 import BeautifulSoup
from prettytable import PrettyTable
from translate import Translator

from FBRank.utils.exceptions import IllegalArgumentException, NotSupprotedYetException
from FBRank.utils.utils import league_news_pattern,EPL_League_transformat

league_news_pattern_target = re.compile(league_news_pattern)
translator= Translator(from_lang="zh_CN",to_lang="en")

def parse_league_rank(url, index=0):
    """
    :param url: weburl which contains ceatain league rank information
    :return: PrettyTable Object contains league rank

    tips:
    - origin premire leagure website will be instability
    - sports tencent return JSON Data,But cannot be directy play(with out front-end)
    url = "http://matchweb.sports.qq.com/team/rank"
    payloads = 
         'callback': 'recommendlist',
         'competitionId': '8',
         'from': 'sporthp',
         '_': str(randint(1, 100000))
     }
    response = requests.get(url, params=payloads).content.decode('utf-8')
    """

    cur = 1
    table = PrettyTable(["Position", "Club", "Player", "Won", "Drawn", "Lost", "GF", "GA",
                         "GD", "AGF", "AGA", "AGD", "APoints", "Points"])
    soup = BeautifulSoup(requests.get(url).content.decode("utf-8"), "lxml")
    for t in soup.find_all("tr", class_=re.compile(r"trbg[red|yellow|blue|grey]")):
        club = [c.get_text() for c in t.find_all("td") if c.get_text()]
        club[1]=EPL_League_transformat.get(club[1],'UnKnown')
        if (cur == index):
            return club
        cur += 1
        table.add_row(club)
    if index != 0:
        raise IllegalArgumentException(
                "index out of range,the max is {} / {}".format(cur, cur))
    return table


def parse_league_news(url):
    """
    :param url: weburl which contains ceatain league news information
    :return: dict {index:[title,url]}
    """
    if url.endswith('bundesliga/'):
        raise NotSupprotedYetException("bundesliga news still not to be exported,wait for the next version:-D")
    news_dict = defaultdict(list)
    web_news = BeautifulSoup(requests.get(url).content.decode('utf-8'), 'lxml')
    key = 1
    for news in web_news.find('div', class_='banner_list slide').find_all('h3')[:-1]:
        # still to be optimized
        title = re.search(r'title="(.*?)"', str(news)).group(1)
        url = re.search(r'href="(.*?)"', str(news)).group(1)
        # use str,because what people input will be str
        news_dict[str(key)].extend([url, title])
        key += 1
    return news_dict


def show_news(news_dict):
    """
    :param news_dict: news_dict returned from parse_league_news
    :return: None,just print
    """
    table = PrettyTable(["ID", "link"])
    for id, (_, title) in news_dict.items():
        table.add_row([id, translator.translate(title)])
    print(table)
    while True:
        prompt = input('------input `ID` to get specific content ，or input `q` to quit------\n')
        if prompt == 'q':
            return 'done.'
        elif prompt not in news_dict.keys():
            print('Please input any ID inside the table')
        else:
            print(get_news_from_index(news_dict[prompt][0]))


def get_news_from_index(url):
    """
    get wen news from certain url
    :param url: new url
    :return: news content,plain text
    """
    headers = {
        'Accept': 'image/webp,image/*,*/*;q=0.8',
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'zh',
        'Cache-Control': 'max-age=0',
        'Connection': 'keep-alive',
        'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'
    }
    soup = BeautifulSoup(requests.get(url).content.decode("gb2312"), "lxml")
    return translator.translate(soup.find("dl").find_all("dd")[4].get_text())



# -*- coding:utf-8 -*-

"""
Entry Point Enterration
"""
from __future__ import print_function
import argparse
import sys
from os import path

sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

from FBRank.object.League import League


# from FBRank.utils.utils import PY2

def execute():
    parser = argparse.ArgumentParser(description="load football list")
    parser.add_argument("-l", "--league", dest="league")
    parser.add_argument("ask", default="rank", choices=["rank", "info", "news"])
    args = parser.parse_args()

    # deal with league
    league_name, ask_name = vars(args).get('league'), vars(args).get('ask')
    league = League(league_name)
    if (ask_name == 'rank'):
        print(league.rank)
    elif (ask_name == 'news'):
        print(league.news)


if __name__ == '__main__':
    execute()



# -*- coding:utf-8
"""
Use For All League
"""

from FBRank.parse.League import parse_league_rank, parse_league_news, show_news
from FBRank.utils.utils import league_configure, league_transformat
from FBRank.utils.exceptions import IllegalNameException


class League(object):
    def __init__(self, intro_name):
        self.name = self._get_name(intro_name)  # to be converted
        self.rank = self._get_webrank()

    @property
    def news(self):
        return self._get_news()

    def _get_name(self, pass_name):
        for league_list in league_transformat.keys():
            if pass_name in league_list or pass_name.lower() in league_list:
                return league_transformat.get(league_list)

        raise IllegalNameException("")

    def _get_news(self):
        news_dict = parse_league_news(self._get_news_url())
        show_news(news_dict)

    def _get_news_url(self):
        return league_configure[self.name]['news_url']

    def _get_webrank(self, index=0):
        return parse_league_rank(self._get_webrank_url(), index)

    def _get_webrank_url(self):
        return league_configure[self.name]['rank_url']

    def __repr__(self):
        return '<League {}>'.format(self.name)
